/*
 * AnyaryOperationAbstraction.hpp
 *
 *  Created on: 20. 8. 2017
 *	  Author: Jan Travnicek
 */

#include "AnyaryOperationAbstraction.hpp"

namespace abstraction {

bool AnyaryOperationAbstractionImpl::attachInput ( const std::shared_ptr < OperationAbstraction > & input, size_t index, bool move, bool checkInput ) {
	if ( input == nullptr )
		return false;

	if ( checkInput && ! this->checkInput ( input, 0 /* Note: yes index zero */ ) )
		return false;

	if ( m_params.size ( ) < index + 1 )
		m_params.resize ( index + 1 );

	m_params [ index ] = std::make_pair ( input, move );
	return true;
}

bool AnyaryOperationAbstractionImpl::detachInput ( size_t index ) {
	if ( index >= m_params.size ( ) )
		throw std::invalid_argument ( "Parameter index " + ext::to_string ( index ) + " out of bounds.");

	m_params [ index ].first = nullptr;
	m_params [ index ].second = false;

	return true;
}

bool AnyaryOperationAbstractionImpl::inputsAttached ( ) const {
	auto attached_lambda = [ ] ( const std::pair < std::shared_ptr < OperationAbstraction >, bool > & param ) {
		return ( bool ) param.first;
	};

	return std::all_of ( m_params.begin ( ), m_params.end ( ), attached_lambda );
}

bool AnyaryOperationAbstractionImpl::eval ( ) {
	if ( ! inputsAttached ( ) )
		return false;

	if ( this->evaluated ( ) )
		return true;

	for ( const std::pair < std::shared_ptr < OperationAbstraction >, bool > & param : m_params )
		if ( ! param.first->eval ( ) )
			return false;

	return this->run ( );
}

} /* namespace abstraction */
