/*
 * ValueOperationAbstraction.hpp
 *
 *  Created on: 11. 7. 2017
 *	  Author: Jan Travnicek
 */

#ifndef _RETURN_VALUE_OPERATION_ABSTRACTION_HPP_
#define _RETURN_VALUE_OPERATION_ABSTRACTION_HPP_

#include <memory>

#include <common/ParamQualifiers.hpp>

#include <abstraction/ValueOperationAbstraction.hpp>
#include <abstraction/VariableOperationAbstraction.hpp>

namespace abstraction {

template < class ReturnType >
class ReturnValueOperationAbstraction : virtual public ValueOperationAbstraction < ReturnType > {
public:
	bool isAutoMove ( ) const override {
		return true;
	}

	std::shared_ptr < abstraction::OperationAbstraction > getVariableOperationAbstraction ( ) override {
		std::shared_ptr < abstraction::OperationAbstraction > res = std::make_shared < abstraction::VariableOperationAbstraction < ReturnType > > ( );
		res->attachInput ( this->getProxyAbstraction ( ), 0, false, false );
		return res;
	}
};

template < >
class ReturnValueOperationAbstraction < void > : virtual public OperationAbstraction {
public:
	template < typename ... ParamTypes, typename Callable >
	inline void run_helper ( Callable callback, const ext::array < std::pair < std::shared_ptr < OperationAbstraction >, bool >, sizeof ... ( ParamTypes ) > & inputs ) {
		abstraction::apply < ParamTypes ... > ( callback, inputs );
	}

	ext::type_index getReturnTypeIndex ( ) const override;

	ext::set < abstraction::ParamQualifiers::ParamQualifier > getReturnTypeQualifiers ( ) const override;

	bool evaluated ( ) const override;

	std::shared_ptr < abstraction::OperationAbstraction > getVariableOperationAbstraction ( ) override;
};

} /* namespace abstraction */

#endif /* _RETURN_VALUE_OPERATION_ABSTRACTION_HPP_ */
