/*
 * ValueOperationAbstraction.hpp
 *
 *  Created on: 11. 7. 2017
 *	  Author: Jan Travnicek
 */

#ifndef _VALUE_OPERATION_ABSTRACTION_HPP_
#define _VALUE_OPERATION_ABSTRACTION_HPP_

#include <alib/array>
#include <alib/memory>

#include <common/ParamQualifiers.hpp>
#include <common/AbstractionHelpers.hpp>

#include <abstraction/ValueInterface.hpp>

namespace abstraction {

template < class ReturnType >
class ValueOperationAbstractionImpl : virtual public ValueInterface < ReturnType > {
	mutable std::optional < ReturnType > m_data;

protected:
	void setData ( ReturnType && data ) {
		m_data = std::move ( data );
	}

	ReturnType && getValue ( ) const override {
		return std::move ( m_data.value ( ) );
	}

public:
	template < typename ... ParamTypes, typename Callable >
	inline void run_helper ( Callable callback, const ext::array < std::pair < std::shared_ptr < OperationAbstraction >, bool >, sizeof ... ( ParamTypes ) > & inputs ) {
		if ( ! evaluated ( ) )
			setData ( abstraction::apply < ParamTypes ... > ( callback, inputs ) );
	}

	bool evaluated ( ) const override {
		return ( bool ) m_data;
	}
};

template < class ReturnType >
class ValueOperationAbstractionImpl < const ReturnType > : virtual public ValueInterface < ReturnType > {
	mutable std::optional < ReturnType > m_data;

protected:
	void setData ( ReturnType && data ) {
		m_data = std::move ( data );
	}

	ReturnType && getValue ( ) const override {
		return std::move ( m_data.value ( ) );
	}

public:
	template < typename ... ParamTypes, typename Callable >
	inline void run_helper ( Callable callback, const ext::array < std::pair < std::shared_ptr < OperationAbstraction >, bool >, sizeof ... ( ParamTypes ) > & inputs ) {
		if ( ! evaluated ( ) )
			setData ( abstraction::apply < ParamTypes ... > ( callback, inputs ) );
	}

	bool evaluated ( ) const override {
		return ( bool ) m_data;
	}
};

template < class ReturnType >
class ValueOperationAbstractionImpl < ReturnType & > : virtual public ValueInterface < ReturnType > {
	mutable std::optional < std::reference_wrapper < ReturnType > > m_data;

protected:
	void setData ( ReturnType & data ) {
		m_data = std::reference_wrapper < ReturnType > ( data );
	}

	ReturnType && getValue ( ) const override {
		return std::move ( m_data->get ( ) );
	}

public:
	template < typename ... ParamTypes, typename Callable >
	inline void run_helper ( Callable callback, const ext::array < std::pair < std::shared_ptr < OperationAbstraction >, bool >, sizeof ... ( ParamTypes ) > & inputs ) {
		if ( ! evaluated ( ) )
			setData ( abstraction::apply < ParamTypes ... > ( callback, inputs ) );
	}

	bool evaluated ( ) const override {
		return ( bool ) m_data;
	}
};

template < class ReturnType >
class ValueOperationAbstractionImpl < const ReturnType & > : virtual public ValueInterface < ReturnType > {
	mutable std::optional < std::reference_wrapper < ReturnType > > m_data;

protected:
	void setData ( const ReturnType & data ) {
		m_data = std::reference_wrapper < ReturnType > ( const_cast < ReturnType & > ( data ) );
	}

	ReturnType && getValue ( ) const override {
		return std::move ( m_data->get ( ) );
	}

public:
	template < typename ... ParamTypes, typename Callable >
	inline void run_helper ( Callable callback, const ext::array < std::pair < std::shared_ptr < OperationAbstraction >, bool >, sizeof ... ( ParamTypes ) > & inputs ) {
		if ( ! evaluated ( ) )
			setData ( abstraction::apply < ParamTypes ... > ( callback, inputs ) );
	}

	bool evaluated ( ) const override {
		return ( bool ) m_data;
	}
};

template < class ReturnType >
class ValueOperationAbstractionImpl < ReturnType && > : virtual public ValueInterface < ReturnType > {
	mutable std::optional < std::reference_wrapper < ReturnType > > m_data;

protected:
	void setData ( ReturnType && data ) {
		m_data = std::reference_wrapper < ReturnType > ( data );
	}

	ReturnType && getValue ( ) const override {
		return std::move ( m_data->get ( ) );
	}

public:
	template < typename ... ParamTypes, typename Callable >
	inline void run_helper ( Callable callback, const ext::array < std::pair < std::shared_ptr < OperationAbstraction >, bool >, sizeof ... ( ParamTypes ) > & inputs ) {
		if ( ! evaluated ( ) )
			setData ( abstraction::apply < ParamTypes ... > ( callback, inputs ) );
	}

	bool evaluated ( ) const override {
		return ( bool ) m_data;
	}
};

template < class ReturnType >
class ValueOperationAbstractionImpl < const ReturnType && > : virtual public ValueInterface < ReturnType > {
	mutable std::optional < std::reference_wrapper < ReturnType > > m_data;

protected:
	void setData ( const ReturnType && data ) {
		m_data = std::reference_wrapper < ReturnType > ( const_cast < ReturnType & > ( data ) );
	}

	ReturnType && getValue ( ) const override {
		return std::move ( m_data->get ( ) );
	}

public:
	template < typename ... ParamTypes, typename Callable >
	inline void run_helper ( Callable callback, const ext::array < std::pair < std::shared_ptr < OperationAbstraction >, bool >, sizeof ... ( ParamTypes ) > & inputs ) {
		if ( ! evaluated ( ) )
			setData ( abstraction::apply < ParamTypes ... > ( callback, inputs ) );
	}

	bool evaluated ( ) const override {
		return ( bool ) m_data;
	}
};

template < class ReturnType >
class ValueOperationAbstraction : virtual public ValueOperationAbstractionImpl < ReturnType > {
	bool isConst ( ) const override {
		return std::is_const_v < std::remove_reference_t < ReturnType > >;
	}

	bool isRvalueRef ( ) const override {
		return std::is_rvalue_reference_v < ReturnType >;
	}

	bool isLvalueRef ( ) const override {
		return std::is_lvalue_reference_v < ReturnType >;
	}

public:
	ext::set < abstraction::ParamQualifiers::ParamQualifier > getReturnTypeQualifiers ( ) const override {
		return abstraction::ParamQualifiers::paramQualifiers < ReturnType > ( );
	}
};

} /* namespace abstraction */

#endif /* _VALUE_OPERATION_ABSTRACTION_HPP_ */
